indexing

	description:

		"Objects that output a sequence of XM_PATH_ITEMs."

	library: "Gobo Eiffel XSLT Library"
	copyright: "Copyright (c) 2004, Colin Adams and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2005/06/29 15:07:16 $"
	revision: "$Revision: 1.12 $"

class XM_XSLT_SEQUENCE_OUTPUTTER	

inherit
	
	XM_XSLT_OUTPUTTER	
		redefine
			start_document
		end

	XM_XPATH_STANDARD_NAMESPACES

	XM_XPATH_ERROR_TYPES

	XM_XPATH_AXIS

	XM_XPATH_DEBUGGING_ROUTINES

	-- This outputter is used when writing a sequence of atomic values and nodes,
	--  e.g. when xsl:variable is used with content and an "as" attribute. The outputter
	--  builds the sequence and provides access to it. (It isn't really an outputter at all,
	--  it doesn't pass the events to anyone, it merely constructs the sequence in memory
	--  and provides access to it). Note that the event sequence can include calls such as
	--  start_element and end_element that require trees to be built. If nodes such as attributes
	--  and text nodes are received while an element is being constructed, the nodes are added
	--  to the tree. Otherwise, "orphan" nodes (nodes with no parent) are created and added
	--  directly to the sequence.
 
	-- This class is not used to build temporary trees. For that, the XM_XSLT_COMPLEX_CONTENT_OUTPUTTER is used.

creation

	make, make_with_size

feature {NONE} -- Initialization

	make (a_transformer: XM_XSLT_TRANSFORMER) is
			-- Establish invariant.
		require
			transformer_not_void: a_transformer /= Void
		do
			create output_list.make_default
			system_id := ""
			transformer := a_transformer
		ensure
			transformer_set: transformer = a_transformer
		end

	make_with_size (a_size: INTEGER; a_transformer: XM_XSLT_TRANSFORMER) is
			-- Establish invariant.
		require
			strictly_positive_size: a_size > 0
			transformer_not_void: a_transformer /= Void
		do
			create output_list.make (a_size)
			system_id := ""
			transformer := a_transformer
		ensure
			transformer_set: transformer = a_transformer
		end

feature -- Access

	transformer: XM_XSLT_TRANSFORMER
			-- Transformer for error reporting

	is_under_construction: BOOLEAN
			-- Is `sequence' under construction?

	sequence: XM_XPATH_VALUE is
			-- Built sequence
		require
			not_under_construction: not is_under_construction
		do
			if cached_sequence = Void then
				inspect
					output_list.count
				when 0 then
					create {XM_XPATH_EMPTY_SEQUENCE} cached_sequence.make
				when 1 then
					cached_sequence := output_list.item (1).as_item_value
				else
					create {XM_XPATH_SEQUENCE_EXTENT} cached_sequence.make_from_list (output_list)
				end
			end
			Result := cached_sequence
		ensure
			sequence_not_void: Result /= Void
		end

	first_item: XM_XPATH_ITEM is
			-- First item of output sequence
		do
			if output_list.count > 0 then
				Result := output_list.item (1)
			end
		end

feature -- Events

	on_error (a_message: STRING) is
			-- Event producer detected an error.
		local
			an_error: XM_XPATH_ERROR_VALUE
			a_uri, a_code, a_text: STRING
			an_index, a_second_index: INTEGER
		do
			if tree /= Void then
				tree.on_error (a_message)
			else
				an_index := a_message.index_of (':', 1)
				if a_message.count > an_index + 1 and then STRING_.same_string (a_message.substring (1, 1), "X") and then an_index > 0 then
					if STRING_.same_string (a_message.substring (1, 2), "XT") then
						a_uri := ""
					else
						a_uri := Xpath_errors_uri
					end
					a_code := a_message.substring (1, an_index - 1)
					a_text := a_message.substring (an_index + 2, a_message.count)
				elseif a_message.count > 0 and then STRING_.same_string (a_message.substring (1, 1), "{") then
					an_index := a_message.index_of ('}', 2)
					check
						closing_brace_found: an_index > 1
					end
					a_uri := a_message.substring (2, an_index - 1)
					a_second_index := a_message.index_of (':', an_index + 1)
					check
						colon_found: a_second_index > an_index + 1
					end
					a_code := a_message.substring (an_index + 1, a_second_index - 1)
					STRING_.left_adjust (a_code)
					STRING_.right_adjust (a_code)
					a_text := a_message.substring (a_second_index + 1, a_message.count)
				else
					a_text := a_message
					a_uri := Gexslt_eiffel_type_uri
					a_code := "SERIALIZATION_ERROR"
				end
				create an_error.make_from_string (a_text, a_uri, a_code, Dynamic_error)
				transformer.report_fatal_error (an_error, Void)
			end
		end

	start_document is
			-- New document
		do
			cached_sequence := Void
			is_under_construction := True
			is_document_started := True
		end

	start_element (a_name_code: INTEGER; a_type_code: INTEGER; properties: INTEGER) is
			-- Notify the start of an element.
		local
			a_reducer: XM_XSLT_NAMESPACE_REDUCER
			a_complex_outputter: XM_XSLT_COMPLEX_CONTENT_OUTPUTTER
			a_node_factory: XM_XPATH_NODE_FACTORY
		do
			if in_start_tag then
				start_content
			end
			if tree = Void then
				create a_node_factory
				create builder.make (a_node_factory)
				create a_reducer.make (builder)
				create a_complex_outputter.make (a_reducer)
				tree := a_complex_outputter
				tree.set_system_id (system_id)
				tree.start_document
				tree.start_element (a_name_code, a_type_code, properties)
			else
				tree.start_element (a_name_code, a_type_code, properties)
			end
			level := level + 1
			in_start_tag := True
			previous_atomic := False
		end

	end_element is
			-- Notify the end of an element.
		local
			a_document: XM_XPATH_TREE_DOCUMENT
		do
			if in_start_tag then
				start_content
			end
			level := level - 1
			if level = 0 then
				tree.end_element
				tree.end_document
				a_document := builder.tree_document
				tree := Void
				builder := Void
				append_item (a_document.document_element)

				-- Now free the document from memory

				shared_name_pool.remove_document_from_pool (a_document.document_number)
			else
				tree.end_element
			end
			previous_atomic := False
		end

	notify_namespace (a_namespace_code: INTEGER; properties: INTEGER) is
			-- Notify a namespace.
		do
			if tree = Void then
				todo ("notify_namespace", true)
			else
				tree.notify_namespace (a_namespace_code, properties)
			end
			previous_atomic := False
		end

	notify_attribute (a_name_code: INTEGER; a_type_code: INTEGER; a_value: STRING; properties: INTEGER) is
			-- Notify an attribute.
		do
			if tree = Void then
				todo ("notify_attribute", true)
			else
				tree.notify_attribute (a_name_code, a_type_code, a_value, properties)
			end
			previous_atomic := False
		end

	start_content is
			-- Notify the start of the content, that is, the completion of all attributes and namespaces.
		do
			in_start_tag := False
			tree.start_content
			previous_atomic := False
		end

	notify_characters (chars: STRING; properties: INTEGER) is
			-- Notify character data.
		local
			an_orphan: XM_XPATH_ORPHAN
		do
			if chars.count /= 0 then
				if in_start_tag then start_content end
				if tree = Void then
					create an_orphan.make (Text_node, chars)
					append_item (an_orphan)
				else
					tree.notify_characters (chars, properties)
				end
				previous_atomic := False
			end
		end

	notify_comment (a_content_string: STRING; properties: INTEGER) is
			-- Notify a comment.
		do
			if in_start_tag then start_content end
			if tree = Void then
					todo ("notify_comment", true)
				else
					tree.notify_comment (a_content_string, properties)
			end
			previous_atomic := False
		end

	notify_processing_instruction (a_name: STRING; a_data_string: STRING; properties: INTEGER) is
			-- Notify a processing instruction.
		do
			if in_start_tag then start_content end
			if tree = Void then
					todo ("notify_comment", true)
				else
					tree.notify_processing_instruction (a_name, a_data_string, properties)
			end
			previous_atomic := False
		end

	end_document is
			-- Notify the end of the document.
		do
			previous_atomic := False
			is_under_construction := False
		end

	append_item (an_item: XM_XPATH_ITEM) is
			-- Output an item (atomic value or node) to the sequence.
		do

			-- If an atomic value is written to a tree, and the previous item was also
			--  an atomic value, then add a single space to separate them

			if previous_atomic and then tree /= Void and then an_item.is_atomic_value then
				tree.notify_characters (" ", 0)
			end
			previous_atomic := an_item.is_atomic_value
			if tree = Void then
				output_list.force_last (an_item)
			elseif an_item.is_atomic_value then
				tree.notify_characters (an_item.as_atomic_value.string_value, 0)
			else
				check
					node: an_item.is_node
					-- Items are atomic values or nodes
				end
				an_item.as_node.copy_node (tree, All_namespaces, True)
			end
		end

feature -- Element change

	set_system_id (a_system_id: STRING) is
			-- Set the system-id of the destination tree.
		do
			system_id := a_system_id
		ensure then
			system_id_set: system_id = a_system_id
		end

	set_document_locator (a_locator: XM_XPATH_LOCATOR) is
			-- Set the locator.
		do
			-- do nothing
		end

feature {NONE} -- Implementation

	output_list: DS_ARRAYED_LIST [XM_XPATH_ITEM]
			-- Output list

	in_start_tag: BOOLEAN
			-- Are we within an element?

	tree: XM_XPATH_RECEIVER
			-- Output tree

	builder: XM_XPATH_TREE_BUILDER
			-- Tree builder

	level: INTEGER
			-- Element nesting level

	cached_sequence: XM_XPATH_VALUE
			-- Result returned by `sequence'

invariant

	output_list_not_void: output_list /= Void
	transformer_not_void: transformer /= Void

end
	
